﻿using System;
using System.Windows.Forms;
using System.ComponentModel;
using iTextSharp.text;
using iTextSharp.text.pdf;
using System.IO;

namespace PDF_R2L
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // reset menu when form loaded
        private void Form1_Load(object sender, EventArgs e)
        {
            execForm(false);
            progressBar1.Value = 0;
        }

        // change form menu
        private void execForm(bool status)
        {
            if(status == false)
            {
                btnSrcFolder.Enabled = true;
                btnDestFolder.Enabled = true;
                btnR2L.Enabled = true;
                btnAbort.Enabled = false;
                btnExit.Enabled = true;
                progressBar1.Visible = false;
                label3.Text = "処理中ファイル...";
            }
            else
            {
                btnSrcFolder.Enabled = false;
                btnDestFolder.Enabled = false;
                btnR2L.Enabled = false;
                btnAbort.Enabled = true;
                btnExit.Enabled = false;
                progressBar1.Visible = true;
                label3.Text = string.Empty;
            }
        }

        // set folder using drag and drop
        private void tbSrcFolder_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop)==true)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void tbSrcFolder_DragDrop(object sender, DragEventArgs e)
        {
            string[] srcFolder = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            if (srcFolder.Length <= 0)
            {
                return;
            }
            else
            {
                tbSrcFolder.Text = srcFolder[0];
                fbdSrc.SelectedPath = srcFolder[0];
            }
        }

        private void tbDestFolder_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop)==true)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void tbDestFolder_DragDrop(object sender, DragEventArgs e)
        {
            string[] dstFolder = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            if(dstFolder.Length <= 0)
            {
                return;
            }
            else
            {
                tbDestFolder.Text = dstFolder[0];
                fbdDest.SelectedPath = dstFolder[0];
            }
        }

        // set folder using folderbrowserdialog
        private void btnSrcFolder_Click(object sender, EventArgs e)
        {
            if (fbdSrc.ShowDialog() == DialogResult.OK)
            {
                tbSrcFolder.Text = fbdSrc.SelectedPath;
            }
        }

        private void btnDestFolder_Click(object sender, EventArgs e)
        {
            if (fbdDest.ShowDialog() == DialogResult.OK)
            {
                tbDestFolder.Text = fbdDest.SelectedPath;
            }
        }

        // set folder by text input
        private void tbSrcFolder_TextChanged(object sender, EventArgs e)
        {
            fbdSrc.SelectedPath = tbSrcFolder.Text;
        }

        private void tbDestFolder_TextChanged(object sender, EventArgs e)
        {
            fbdDest.SelectedPath = tbDestFolder.Text;
        }

        // check folder existance before execute
        private bool checkBeforeStart()
        {
            var errmsg = string.Empty;
            if (Directory.Exists(fbdSrc.SelectedPath) == false) errmsg += "作業対象フォルダが存在しません。";
            if (Directory.Exists(fbdDest.SelectedPath) == false) errmsg += "出力フォルダが存在しません。";
            if (fbdSrc.SelectedPath == fbdDest.SelectedPath) errmsg += "同じフォルダは指定できません。";
            if (errmsg.Length == 0)
            {
                return true;
            }
            else
            {
                MessageBox.Show(errmsg);
                return false;
            }
        }

        // backgroundworker
        private void bw_DoWork(object sender, DoWorkEventArgs e)
        {
            string srcFo = fbdSrc.SelectedPath;
            string dstFo = fbdDest.SelectedPath;
            string newFo = string.Empty;

            string[] files = Directory.GetFiles(srcFo, "*.pdf", SearchOption.AllDirectories);
            int totalFile = files.Length;
            int fileCount = 0;
            int percentage = 0;

            foreach (string f in files)
            {
                if (bw.CancellationPending)
                {
                    e.Cancel = true;
                    return;
                }
                newFo = dstFo + (Path.GetDirectoryName(f) + @"\").Replace(srcFo, string.Empty);
                if (!Directory.Exists(newFo)) Directory.CreateDirectory(newFo);
                using (PdfReader pdfr = new PdfReader(f))
                {
                    PdfStamper stamp = new PdfStamper(pdfr, new FileStream(newFo + @"\" + Path.GetFileNameWithoutExtension(f) + "【右綴じ】" + @".pdf", FileMode.Create));
                    stamp.AddViewerPreference(PdfName.DIRECTION, PdfName.R2L);
                    stamp.Dispose();
                    pdfr.Dispose();
                }
                fileCount++;
                percentage = fileCount * 100 / totalFile;
                bw.ReportProgress(percentage);
            }
        }

        // click exec button
        private void btnR2L_Click(object sender, EventArgs e)
        {
            while (checkBeforeStart() == false) return;
            execForm(true);
            bw.RunWorkerAsync();
        }

        // refresh progressbar
        private void bw_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            progressBar1.Value = e.ProgressPercentage;
        }

        // completion of backgroundworker
        private void bw_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progressBar1.Value = 0;
            if (e.Error != null)
            {
                MessageBox.Show("エラーが発生しました。");
            }
            else
            {
                if (e.Cancelled)
                {
                    MessageBox.Show("キャンセルされました。");
                }
                else
                {
                    MessageBox.Show("終了しました。");
                }
            }
            execForm(false);
        }

        // click abort button
        private void btnAbort_Click(object sender, EventArgs e)
        {
            if (bw.IsBusy) bw.CancelAsync();
        }

        // click program exit button
        private void btnExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
    }
}
